/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.filetransferbc;

import org.objectweb.petals.common.descriptor.Extensions;
import org.objectweb.petals.component.common.util.PetalsExtensionsUtil;

/**
 * A singleton used to analyze service unit extensions.
 *
 * @author Christophe HAMERLING (chamerling) - eBM WebSourcing
 *
 */
public class ServiceUnitExtensionsAnalyzer {

    /**
     * Get instance
     *
     * @return
     */
    public static final ServiceUnitExtensionsAnalyzer getInstance() {
        if (INSTANCE == null) {
            INSTANCE = new ServiceUnitExtensionsAnalyzer();
        }
        return INSTANCE;
    }

    public static ServiceUnitExtensionsAnalyzer INSTANCE = null;

    private static final String ADD_TAGS = "add-tags";

    private static final String FILE_NAME = "file-name";

    private static final String PATTERN = "pattern";

    public static final String POLLING_PERIOD = "polling-perod";

    public static final String OUTPUT_DIR = "output-dir";

    public static final String FILENAME_FILTER = "filename-filter";

    public static final String REMOVE_TAGS = "remove-tags";

    public static final String ATTACHMENTS = "attachments";

    public static final String SCRIPT = "script";

    /**
     * Creates a new instance of {@link ServiceUnitExtensionsAnalyzer}
     *
     */
    private ServiceUnitExtensionsAnalyzer() {
    }

    /**
     * Retrieve pooling period from the extensions. If polling is not found, the
     * default value is set to 2000 ms.
     *
     * @param extensions
     * @return the polling period in ms
     */
    public long retrievePollingPeriod(Extensions extensions) {
        long polling = 2000;

        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, ServiceUnitExtensionsAnalyzer.POLLING_PERIOD);
        if (tmp != null) {
            try {
                polling = Long.parseLong(tmp);
            } catch (NumberFormatException e) {
            }
        }
        return polling;
    }

    /**
     * Retrieve the output directory from the consumes bean. If the output
     * directory has not been set, set the default directory to the component
     * installation root.
     *
     */
    public String retrieveOutputDir(Extensions extensions) {
        return PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, OUTPUT_DIR);
    }

    /**
     *
     * @param extensions
     * @return
     */
    public String[] retrieveRemoveTags(Extensions extensions) {
        String result[] = null;
        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, REMOVE_TAGS);

        if (tmp != null && tmp.length() > 0) {
            result = tmp.split(" ");
        }
        return result;

    }

    /**
     *
     * @param extensions
     * @return
     */
    public String retrieveFileName(Extensions extensions) {
        return PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, FILE_NAME);
    }

    /**
     *
     * @param extensions
     * @return
     */
    public boolean isAttachmentMode(Extensions extensions) {
        boolean result = false;
        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, ATTACHMENTS);
        if (tmp != null && tmp.length() > 0) {
            result = Boolean.parseBoolean(tmp);
        }
        return result;
    }

    /**
     *
     * @param extensions
     * @return
     */
    public String[] retrieveAddTags(Extensions extensions) {
        String[] result = null;
        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, ADD_TAGS);
        if (tmp != null && tmp.length() > 0) {
            result = tmp.split(" ");
        }
        return result;
    }

    /**
     * Retrieve the MEP
     *
     * @param extensions
     * @return
     */
    public String retrieveMEP(Extensions extensions) {
        return PetalsExtensionsUtil.extractValueFromKeyValueExtension(extensions,
                PATTERN, "");
    }

    /**
     * Get the file filter value from the extensions. This filter will be used
     * to filter the files to handle in the polling directory.
     *
     * @return the file filter or empty string if not found
     */
    public String retrieveFileNameFilter(Extensions extensions) {
        String filter = "";
        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, FILENAME_FILTER);
        if (tmp != null) {
            filter = tmp;
        }

        return filter;
    }

    /**
     * Retrieve the script value
     *
     * @param extensions
     * @return
     */
    public String retrieveScript(Extensions extensions) {
        String script = "";
        String tmp = PetalsExtensionsUtil.extractValueFromKeyValueExtension(
                extensions, SCRIPT);
        if (tmp != null) {
            script = tmp;
        }

        return script;
    }

}
