/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * Id : ${file_name} 154 ${date}  rbarraza
 * -------------------------------------------------------------------------
 */

package org.objectweb.petals.binding.filetransferbc.listeners;

import java.io.File;
import java.io.IOException;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.jbi.messaging.NormalizedMessage;
import javax.xml.transform.TransformerException;

import org.objectweb.petals.binding.filetransferbc.FileTransferBCException;
import org.objectweb.petals.binding.filetransferbc.IOUtils;
import org.objectweb.petals.binding.filetransferbc.ServiceUnitExtensionsAnalyzer;
import org.objectweb.petals.common.descriptor.Extensions;
import org.objectweb.petals.common.util.StringHelper;
import org.objectweb.petals.common.util.XMLUtil;
import org.objectweb.petals.component.common.HandlingException;
import org.objectweb.petals.component.common.PEtALSComponentSDKException;
import org.objectweb.petals.component.common.bc.JBIListener;
import org.objectweb.petals.component.common.util.MessageExchangeWrapper;
import org.objectweb.petals.component.common.util.SourceHelper;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

/**
 * The file transfer BC JBI listener. Writes a file to hard disk on incoming JBI
 * message.
 *
 * @version $Rev: 250 $Date: {date}
 * @since Petals 1.0
 * @author alouis, Rocio BARRAZA, Christophe HAMERLING - eBMWebsourcing
 *
 */
public class FileTransferBCJBIListener implements JBIListener {

    private static final String DEFAULT_FILE_NAME = "content";

    /**
     * The logger
     */
    private final Logger logger;

    /**
     * The directory path to write files to
     */
    private String destinationDirectoryPath;

    private IOUtils ioUtils = IOUtils.getInstance();

    /**
     * Creates a new instance of {@link FileTransferBCJBIListener}
     *
     * @param rootPath
     * @param logger
     */
    public FileTransferBCJBIListener(String rootPath, Logger logger) {
        this.logger = logger;
        this.destinationDirectoryPath = rootPath;
    }

    /*
     * (non-Javadoc)
     *
     * @see org.objectweb.petals.component.common.bc.JBIListener#onJBIMessage(java.lang.String,
     *      javax.jbi.messaging.MessageExchange,
     *      org.objectweb.petals.tools.jbicommon.descriptor.Extensions)
     */
    public boolean onJBIMessage(String address,
            MessageExchangeWrapper exchange, Extensions extensions)
            throws HandlingException {

        boolean result = false;
        String content = null;
        ServiceUnitExtensionsAnalyzer suAnalyzer = ServiceUnitExtensionsAnalyzer.getInstance();
        NormalizedMessage nm = exchange.getInMessage();

        // get message content as string
        try {
            content = SourceHelper.createString(nm.getContent());
        } catch (PEtALSComponentSDKException e) {
            throw new HandlingException(e.getMessage());
        }

        // Delete content root tags if specified in provides node extensions
        String[] removeTags = suAnalyzer.retrieveRemoveTags(extensions);
        if (removeTags != null) {
            try {
                content = this.deleteRootTags(content, removeTags);
            } catch (TransformerException e) {
                throw new HandlingException(
                        "Problem while unwrapping XML elements from the document.",
                        e);
            }
        }

        // write the content in a file.
        // If the address is not an absolute path, the directory will be
        // expended to the SU install root path.
        File tmpDir = new File(address);
        File parentDir = null;
        if (!tmpDir.isAbsolute()) {
            parentDir = new File(destinationDirectoryPath, address);
        } else {
            parentDir = new File(address);
        }

        File outputDir = parentDir;

        logger.log(Level.INFO, "Writing data to directory : "
                + outputDir.getAbsolutePath());

        if (StringHelper.isNullOrEmpty(content)) {
            logger.log(Level.INFO, "content is null");

        } else {
            // file name : name specified in SU extension (suffixed by the date)
            // or name of the operation
            // or 'content'
            try {
                String fileName = null;
                // search the name in the SU extension
                String prefix = suAnalyzer.retrieveFileName(extensions);
                if (!StringHelper.isNullOrEmpty(prefix)) {
                    fileName = prefix;

                } else {
                    if (!StringHelper.isNullOrEmpty(exchange.getOperation()
                            .getLocalPart())) {
                        fileName = exchange.getOperation().getLocalPart();
                    } else {
                        fileName = DEFAULT_FILE_NAME;
                    }
                }
                ioUtils.writeStringToFile(content, outputDir, fileName);
            } catch (FileTransferBCException e) {
                throw new HandlingException(e);
            }
        }

        // write attachments to files
        if (!nm.getAttachmentNames().isEmpty()) {
            try {
                ioUtils.writeAttachmentsToFiles(nm, outputDir);
            } catch (FileTransferBCException e) {
                throw new HandlingException(e);
            }
        }

        // Call a script if defined in the SU
        String script = suAnalyzer.retrieveScript(extensions);
        if (!StringHelper.isNullOrEmpty(script)) {
            try {
                Runtime.getRuntime().exec(script);
            } catch (IOException e) {
                throw new HandlingException("Can not execute the script : "
                        + script, e);
            }
        }
        return result;
    }

    /**
     * Remove the specified tags in the XML content
     *
     * @param content
     * @param elements
     *            an array of the XML tags to remove, starting from the root
     * @return the body unwrapped, the initial body if tags is null or empty, or
     *         null if a tag was not found in the body
     * @throws TransformerException
     *             XML body is not valid
     */
    private String deleteRootTags(String body, String[] tags)
            throws TransformerException {
        String result = body;
        if (tags != null && tags.length > 0) {
            Document domContent = XMLUtil.createDocumentFromString(body);

            // Retrieve document root element
            Node element = domContent.getFirstChild();
            // remove all elements
            for (int i = 0; i < tags.length && element != null; i++) {
                if (tags[i].equals(element.getNodeName())) {
                    element = XMLUtil.getFirstChild(element);
                } else {
                    element = null;
                }
            }
            if (element != null) {
                result = XMLUtil.createStringFromDOMDocument(element);
            } else {
                result = null;
            }
        }
        return result;
    }
}
